using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WWWings35_DZ.EF;
using WWWings35_GO;
using WWWings35_GO.EF;
using Perseus;

namespace WWWings35_BL.EF
{
 /// <summary>
 /// BL-Manager-Klasse fr Flge
 /// Verwendet im Untergrund Datamanager, der EF einsetzt
 /// </summary>
 public class FlugBLManager
 {
  private FlugDataManager dz = new WWWings35_DZ.EF.FlugDataManager();

  #region Count
  /// <summary>
  /// Anzahl der Flge
  /// </summary>
  /// <returns></returns>
  public long CountFluege()
  {
   return dz.CountFluege();
  }

  /// <summary>
  /// Anzahl der Flge
  /// </summary>
  /// <returns></returns>
  public long CountFluegeVonOrt(string Ort)
  {
   return dz.CountFluegeVonOrt(Ort);
  }

  /// <summary>
  /// Anzahl der Flge auf einer Route
  /// </summary>
  public long CountFluegeRoute(string Abflugort, string Zielort)
  {
   return dz.CountFluegeRoute(Abflugort, Zielort);
  }
  #endregion

  #region Einzelflug
  /// <summary>
  /// Holt einen Flug nach Primrschlssel
  /// </summary
  public Flug GetFlug(long Nr)
  {
   return dz.GetFlug(Nr);
  }

  /// <summary>
  /// Holt einen Flug nach Primrschlssel. Verwendet EntityBag Codeplex Projekt!
  /// </summary>
  public EntityBag<Flug> GetFlug_AsEntityBag(long Nr)
  {
   return dz.GetFlug_AsEntityBag(Nr);
  }
  #endregion

  #region Flugmenge
  /// <summary>
  /// Holt Flge von einem Abflugort
  /// </summary>
  public List<Flug> GetFluegeVonOrt(string Ort)
  {
   return dz.GetFluegeVonOrt(Ort);
  }

  /// <summary>
  /// Holt Flge auf einer Route
  /// </summary>
  public List<Flug> GetFluegeRoute(string Abflugort, string Zielort)
  {
   return dz.GetFluegeRoute(Abflugort, Zielort);
  }

  /// <summary>
  /// Holt Flge  in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeZeitraum(DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return dz.GetFluegeZeitraum(ZeitraumEnde, ZeitraumEnde);
  }


  /// <summary>
  /// Holt Flge auf einer Route in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeRouteZeitraum(string Abflugort, string Zielort, DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return dz.GetFluegeRouteZeitraum(Abflugort, Zielort, ZeitraumStart, ZeitraumEnde);
  }

  /// <summary>
  /// Hollt alle Flge
  /// </summary>
  /// <returns></returns>
  public List<Flug> GetFluege()
  {
   return dz.GetFluege();
  }

  /// <summary>
  /// Holt alle Flge mit Sortieren und Blttern
  /// </summary>
  public List<Flug> GetFluege_Paging(string Sortierung, int Anzahl, int Start)
  {
   return dz.GetFluege_Paging(Sortierung,Anzahl,Start);
  }

  /// <summary>
  /// Holt Flge mit Ortsfilter und Sortieren/Blttern
  /// </summary>
  public List<Flug> GetFluegeVonOrt_Paging(string Ort, string Sortierung, int Anzahl, int Start)
  {
   return dz.GetFluegeVonOrt_Paging(Ort, Sortierung, Anzahl, Start);
  }

  /// <summary>
  /// Holt Flge mit Routenfilter und Sortieren/Blttern
  /// </summary>
  public List<Flug> GetFluegeRoute_Paging(string Abflugort, string Zielort, string Sortierung, int Anzahl, int Start)
  {
   return dz.GetFluegeRoute_Paging(Abflugort, Zielort, Sortierung, Anzahl, Start);
  }
  #endregion

  #region Flge ndern
  /// <summary>
  /// Speichern eines genderten Flugs
  /// </summary>
  /// <param name="FlugNeu"></param>
  /// <returns></returns>
  public bool SaveFlug(Flug FlugNeu)
  {
   return dz.SaveFlug(FlugNeu);
  }


  /// <summary>
  /// Speichern aller genderten und hinzugefgten Flge
  /// </summary>
  public string SaveFluege(List<Flug> fluege)
  {
   return dz.SaveFluege(fluege);
  }

  /// <summary>
  /// Speichern aller nderungen im Kontext
  /// </summary>
  public string SaveFluege()
  {
   return dz.SaveFluege();
  }

  /// <summary>
  /// Fr Desktop-UI: Aufruf, wenn in UI eine Zeile gelscht wurde
  /// </summary>
  public bool MarkFlugForDeletion(Flug flug)
  {
   return dz.MarkFlugForDeletion(flug);
  }



  /// <summary>
  /// Aufruf, wenn in UI eine Zeile ergnzt wurde
  /// </summary>
  public bool MarkFlugForAdding(Flug flug)
  {

   return dz.MarkFlugForAdding(flug);
  }

  #endregion
 }
}
