﻿//----------------------------------------------------------------
// Dieses Beispiel basiert auf dem Beispiel "Advanced Filters" aus den WCF 4.0-Beispielen von Microsoft
//----------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Dispatcher;
using System.Collections;
using System.ServiceModel.Channels;

namespace WCF_Router
{
  /// <summary>
  /// Eigene Filtertable für RoundRobin-Verteilung
  /// </summary>
  /// <typeparam name="TFilterData"></typeparam>
  public class RoundRobinMessageFilterTable<TFilterData> : IMessageFilterTable<TFilterData>
  {
    Dictionary<MessageFilter, TFilterData> filters = new Dictionary<MessageFilter, TFilterData>();


    public RoundRobinMessageFilterTable()
    {
    }

    /// <summary>
    /// Einsprungpunkt: WCF ruft zur Laufzeit diese Methode auf
    /// Bei einem Request-Reply-Router ist genau ein Filter auszuwählen
    /// </summary>
    public bool GetMatchingValue(Message message, out TFilterData value)
    {
      bool outcome;
      RoundRobinAuswahl(message, out value, out outcome);
      return outcome;
    }


    Random Zufall = new Random(DateTime.Now.Millisecond);
    public static int Counter = 0;
    /// <summary>
    /// Die eigentliche Entscheidung auf Basis einer gleichverteilten Auswahl
    /// (Round Robin)
    /// </summary>
    /// <param name="message"></param>
    /// <param name="value"></param>
    /// <param name="outcome"></param>
    private void RoundRobinAuswahl(Message message, out TFilterData value, out bool outcome)
    {
      Console.WriteLine("--> Custom Filter für Nachricht  " + message.Headers.Action + " an " + message.Headers.To.AbsoluteUri.ToString());
      value = default(TFilterData);

      int Anzahlfilter = filters.Count;
      int Auswahl;

     // Round Robin
      Counter++;
      if (Counter > Anzahlfilter - 1) Counter = 0;
      Auswahl = Counter;

     // oder Zufall
      //Auswahl = Zufall.Next(Anzahlfilter);

      // "System.Collections.Generic.List`1[[System.ServiceModel.Description.ServiceEndpoint, System.ServiceModel, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089]]"

      value = filters.Values.ElementAt(Auswahl);
      var e = (value as List<System.ServiceModel.Description.ServiceEndpoint>)[0];

      Console.WriteLine("FILTERTABLE:  Weitergeleitet an: " + e.ListenUri.ToString());

      outcome = true;
    }

    public bool GetMatchingFilter(MessageBuffer messageBuffer, out MessageFilter filter)
    {
      throw new NotImplementedException();
    }

    public bool GetMatchingFilter(Message message, out MessageFilter filter)
    {
      throw new NotImplementedException();
    }

    public bool GetMatchingFilters(MessageBuffer messageBuffer, ICollection<MessageFilter> results)
    {
      throw new NotImplementedException();
    }

    public bool GetMatchingFilters(Message message, ICollection<MessageFilter> results)
    {
      throw new NotImplementedException();
    }


   /// <summary>
    /// Bei aktiviertem Content Filtering
   /// </summary>
    public bool GetMatchingValue(MessageBuffer messageBuffer, out TFilterData value)
    {


     return GetMatchingValue(messageBuffer.CreateMessage(), out value);
    }





   /// <summary>
    /// Bei aktiviertem Content Filtering + Multicast
   /// </summary>

    public bool GetMatchingValues(MessageBuffer messageBuffer, ICollection<TFilterData> results)
    {
     return GetMatchingValues(messageBuffer.CreateMessage(),  results);

    }

   /// <summary>
    /// Bei Multicast
   /// </summary>
    public bool GetMatchingValues(Message message, ICollection<TFilterData> results)
    {
     TFilterData result;
     var match = GetMatchingValue(message, out result);
      results.Add(result);
     return  match;
    }

    // Während Initialisierung: add a message filter to the MessageFilterTable
    public void Add(MessageFilter key, TFilterData value)
    {
      // füge den Filter der Liste hinzu
      this.filters.Add(key, value);
    }


    public bool ContainsKey(MessageFilter key)
    {
      throw new NotImplementedException();
    }

    public ICollection<MessageFilter> Keys
    {
      get { throw new NotImplementedException(); }
    }

    public bool Remove(MessageFilter key)
    {
      throw new NotImplementedException();
    }

    public bool TryGetValue(MessageFilter key, out TFilterData value)
    {
      throw new NotImplementedException();
    }

    public ICollection<TFilterData> Values
    {
      get { throw new NotImplementedException(); }
    }

    public TFilterData this[MessageFilter key]
    {
      get
      {
        throw new NotImplementedException();
      }
      set
      {
        throw new NotImplementedException();
      }
    }

    public void Add(KeyValuePair<MessageFilter, TFilterData> item)
    {
      throw new NotImplementedException();
    }

    public void Clear()
    {
      throw new NotImplementedException();
    }

    public bool Contains(KeyValuePair<MessageFilter, TFilterData> item)
    {
      throw new NotImplementedException();
    }

    public void CopyTo(KeyValuePair<MessageFilter, TFilterData>[] array, int arrayIndex)
    {
      throw new NotImplementedException();
    }

    public int Count
    {
      get { throw new NotImplementedException(); }
    }

    public bool IsReadOnly
    {
      get { throw new NotImplementedException(); }
    }

    public bool Remove(KeyValuePair<MessageFilter, TFilterData> item)
    {
      throw new NotImplementedException();
    }

    public IEnumerator<KeyValuePair<MessageFilter, TFilterData>> GetEnumerator()
    {
      throw new NotImplementedException();
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      throw new NotImplementedException();
    }
  }
}
