using System;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.ServiceModel.Description;
using System.Data.Services;
using System.Linq;

namespace de.WWWings.WCFServer
{

 /// <summary>
 /// WCF-Service Host fr verschiedene World Wide Wings-Dienste
 /// </summary>
 class WWWingsWCFServiceHost
 {
  private long _Counter = 0;

  public long Counter
  {
   get { return _Counter; }
   set
   {
    _Counter = value;
    Console.WriteLine("Counter");
   }
  }

  static System.Collections.Generic.List<ServiceHost> WCFDienste = new System.Collections.Generic.List<ServiceHost>();

  /// <summary>
  /// Start des WCF-Servers
  /// BasePort > 0 -> Alle Portnummer werden um diesen Wert erhht. Ermglicht den mehrmaligen Start dieses Servers auf verschiedenen Ports
  /// </summary>
  public static void StartService(int BasePort = 0)
  {
   // Alternative:
   //NetTcpBinding binding = new NetTcpBinding();

   bool Singleton = false;

   //Uri baseAddress = new Uri("net.tcp://localhost:1234/WWWings/");
   DataServiceHost dsh = new DataServiceHost(typeof(de.WWWings.Dienste.FlugWebDataService), new Uri[] { new Uri("http://localhost:" + (BasePort + 88).ToString() + "/Webservice/WWWingsDataService.svc/") });

   WCFDienste.Add(dsh);
   if (Singleton)
   { // Singleton
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.FlugplanService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.FlugplanServiceEF()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.FlugplanverwaltungService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.BuchungsService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.PassagierverwaltungService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.TestService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.ProtokollService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.PingService()));
    WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.FlugplanServiceREST()));
    //WCFDienste.Add(new ServiceHost(new de.WWWings.Dienste.Zustandsdienst()));
   }
   else
   { // Single-Call bzw. Session
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.FlugplanService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.FlugplanServiceEF)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.FlugplanverwaltungService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.BuchungsService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.PassagierverwaltungService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.TestService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.ProtokollService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.PingService)));
    WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.FlugplanServiceREST)));
    //WCFDienste.Add(new ServiceHost(typeof(de.WWWings.Dienste.Zustandsdienst)));
   }

   // Start aller Dienst in einer Schleife
   foreach (ServiceHost Host in WCFDienste)
   {
    bool Start = true;
    if (Host is DataServiceHost) { Start = false; } // DSH-Dienste dann nicht starten!!!

    // Protokollausgabe
    Print("Starte Dienst " + Host.Description.Name, ConsoleColor.Yellow);
    foreach (System.ServiceModel.Description.ServiceEndpoint e in Host.Description.Endpoints.ToList())
    {
     if (BasePort > 0)
     {
      if (e.Address.Uri.Scheme.Contains("pipe") || e.Address.Uri.Scheme.Contains("msmq")) { Host.Description.Endpoints.Remove(e); continue; } // net-pipe-Dienste dann nicht starten!!!

      var uribuilder = new UriBuilder(e.Address.Uri);
      if (e.Address.Uri.Port > 0) uribuilder.Port = e.Address.Uri.Port + BasePort;
      else uribuilder.Port = 80 + BasePort;
      e.Address = new EndpointAddress(uribuilder.Uri);
     }
     Print(" - Endpunkt: " + e.Address + " (" + e.Binding.Name + ")", ConsoleColor.White);
    }

    try
    {
     if (Start) Host.Open();

    }
    catch (Exception ex)
    {

     Print("FEHLER beim Starten des Dienstes " + Host.Description.Name + ": " + ex.Message, ConsoleColor.Red);
    }



   }


   //// --- Alternative: Code-basierte Konfinguratio
   //// Festlegen der URL
   //Uri URL = new Uri("http://localhost:84/FlugplanserviceCodebasierteKonfiguration");
   //// ServiceHost erzeugen
   //ServiceHost FlugplanService = new ServiceHost(typeof(de.WWWings.Dienste.FlugplanService), URL);
   //// Endpunkt erzeugen
   //FlugplanService.AddServiceEndpoint(typeof(de.WWWings.Dienste.IFlugplanService), new WSHttpBinding(), "FlugplanserviceCodebasierteKonfiguration");
   //// Dienst starten
   //FlugplanService.Open();

   // --- REST-Service Hosting (codebasiert!)
   // Festlegen der URL
   //Uri URL = new Uri("http://localhost:89/");
   //// ServiceHost erzeugen
   //WebServiceHost REST_Host = new WebServiceHost(typeof(de.WWWings.Dienste.FlugplanServiceREST), URL);
   //// Endpunkt erzeugen
   //REST_Host.AddServiceEndpoint(typeof(de.WWWings.Dienste.FlugplanServiceREST), new WebHttpBinding(), "RESTHost");
   //// Dienst starten
   //REST_Host.Open();

  }

  /// <summary>
  /// Beenden des WCF-Servers
  /// </summary>
  internal static void StopService()
  {
   foreach (ServiceHost Host in WCFDienste)
   {
    if (Host.State != CommunicationState.Closed)
     Host.Close();
   }
  }

  /// <summary>
  /// Hilfsmethode fr Protokollierung
  /// </summary>
  private static void Print(string Text, System.ConsoleColor Farbe)
  {
   Console.ForegroundColor = Farbe;
   Console.WriteLine(Text);
   Console.ForegroundColor = ConsoleColor.Gray;
  }
 }
}

