using System;
using System.ServiceModel;
using System.Threading;
using System.Security.Permissions;

namespace de.WWWings.Dienste
{
 [ServiceContract(Namespace = "http://de.ITVisions.Samples.WCF")]
 public interface IPingService
 {

  [OperationContract()]  
  PingInfo Ping();
 
  
 }
 
 // Dienst
 [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession)]
 public class PingService : IPingService
 {
  static Object syncObject = new object();
  static int AnzahlInstanzen;

  int InstanzID;
  int AnzahlAufrufe;

  public PingService()
  {
   //Console.WriteLine("Serverinstanz von TestService erzeugt!");
   lock (syncObject)
   {
    AnzahlInstanzen++;
    InstanzID = AnzahlInstanzen;
    Console.WriteLine("PingService Instanz #{0} erzeugt!", AnzahlInstanzen);
   }
  }

  /// <summary>
  /// Return the InstanceContextMode of the service
  /// </summary>
  /// <returns></returns>
  public string GetInstanceContextMode()
  {
   ServiceHost host = (ServiceHost)OperationContext.Current.Host;
   ServiceBehaviorAttribute behavior = host.Description.Behaviors.Find<ServiceBehaviorAttribute>();
   return behavior.InstanceContextMode.ToString();
  }

  // Impersonifizierung
  [PrincipalPermission(SecurityAction.Demand, Role = "Users")]
  [OperationBehavior(Impersonation = ImpersonationOption.Required)]
  public string GetServerIdentity_Impersonated()
  {   // Return the InstanceContextMode of the service
   string ausgabe = PingInfo.GetIdentity();
   Console.WriteLine("\nIdentittsbestimmung in einer impersonfizierten Operation:");
   Console.WriteLine(ausgabe);
   return ausgabe;
  }

  // Keine Impersonifizierung
  public string GetServerIdentity()
  {   // Return the InstanceContextMode of the service
   string ausgabe = PingInfo.GetIdentity();
   Console.WriteLine("\nIdentittsbestimmung in einer NICHT impersonfizierten Operation:");
   Console.WriteLine(ausgabe);
   return ausgabe;
  }


  public string GetServerName()
  {   // Return the InstanceContextMode of the service
   return System.Environment.MachineName;
  }

  public int GetInstanceId()
  {   // Return the id for this instance
   return InstanzID;
  }

  private int Counter = 0;



  public void AsyncPing(string Client)
  {
   Console.WriteLine("Async-Ping von Client: " + Client);
   // Hole Zeiger auf Implementierung des Server-Proxies fr die Rckrufschnittstelleimplementierung des Clients 
   IPingDuplexCallback client = null;
   client = OperationContext.Current.GetCallbackChannel<IPingDuplexCallback>();
   // Rufe Client 10x auf
   for (int a = 0; a < 10; a++)
   {
    client.ClientPing("Rckruf #" + a + " von " + System.Environment.MachineName);
    System.Threading.Thread.Sleep(1000);
   }
  }


  public PingInfo Ping()
  {
   Console.Write(".");
   lock (this)
   {
    //Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Ping Start...");
    string client = "unbekannt";
    if ((OperationContext.Current != null) && (OperationContext.Current.IncomingMessageHeaders.From != null))
    {
     client = OperationContext.Current.IncomingMessageHeaders.From.ToString() + OperationContext.Current.IncomingMessageHeaders.From.Identity.ToString();
    }

    //Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Ping from " + client);
    PingInfo p = new PingInfo();
    //Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Ping Ende!");
    return p;
   }
  }
  

 }
}

