﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Collections;
using System.Windows.Threading;
using System.Threading;
using System.Collections.ObjectModel;
using de.WWWings.GL_STE;
using de.WWWings.GO_STE;

namespace WWWings_WPF
{
  /// <summary>
  /// Interaction logic for Window1.xaml
  /// </summary>
 public partial class F_Flugverwaltung 
  {
    private void DoEvents()
    {
      if (Application.Current != null)
        Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, new ThreadStart(delegate { }));
    }

    public F_Flugverwaltung()
    {
      InitializeComponent();
  
    }

    private void SetStatus(string s)
    {
      this.C_Status.Content = s;
      DoEvents();
    }

    private void Window_Loaded(object sender, RoutedEventArgs e)
    {

    }

    private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
    {

    }

    #region Variablen
    // DBDirekt
    WWWings6STE DB = new WWWings6STE();
    // BL
    FlugManager BL = new FlugManager();
    // Service
    WPF_CS.AppServer.FlugServiceSTEClient Dienst = new WPF_CS.AppServer.FlugServiceSTEClient();

    // Liste der Flüge - an das Datengitter gebunden
    ObservableCollection<Flug> fluegeAktuelleAnsicht;
    // Kopie der Flüge für Feststellung der gelöschten Flüge
    ObservableCollection<Flug> fluegeKomplett;
    #endregion

    /// <summary>
    /// Laden der Flüge
    /// </summary>
    private void C_Lesen_Click(object sender, RoutedEventArgs e)
    {
      if (this.C_Modus.Text == "") { SetStatus("Keine Datenzugriffsweg gewählt!"); return; }
      // Anzeige löschen    
      this.flugDataGrid.ItemsSource = null;
      // Ausgewählter Ort
      string Ort = this.C_Orte.Text.ToString();
      // Statusanzeige
      SetStatus("Lade über " + this.C_Modus.Text + "...");

      // DBDirekt
      if (this.C_Modus.Text == "DBDirekt")
      {
        fluegeAktuelleAnsicht = new System.Collections.ObjectModel.ObservableCollection<Flug>(from f in DB.Flug where f.Abflugort == Ort select f);
      }
      // BL
      if (this.C_Modus.Text == "BL")
      {
        fluegeAktuelleAnsicht = new System.Collections.ObjectModel.ObservableCollection<Flug>(BL.GetFluege(Ort));
      }
      // Service
      if (this.C_Modus.Text == "Service")
      {
        fluegeAktuelleAnsicht = Dienst.GetFluege(Ort);
      }

      // Registierieren auf Änderungsereignis der Liste (für Anfügen und Löschen von Objekten!)
      fluegeAktuelleAnsicht.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(fluege_CollectionChanged);

      // Aktivieren der Änderungsverfolgung (für DBDirekt und BL wichtig!)
      foreach (Flug f in fluegeAktuelleAnsicht)
      { f.StartTracking(); }

      // Kopie anlegen für Merken der  gelöschten Objekte
      fluegeKomplett = new ObservableCollection<Flug>(fluegeAktuelleAnsicht);

      // Datenbindung an Datengitter
      this.flugDataGrid.ItemsSource = fluegeAktuelleAnsicht;

      // Status setzen
      SetStatus((this.flugDataGrid.ItemsSource as IEnumerable<Flug>).Count() + " Datensätze geladen über " + this.C_Modus.Text + "!");
    }

    /// <summary>
    /// Sonderbehandlung für angefügte und gelöschte Objekte
    /// </summary>
    void fluege_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
    {
      if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
      {
        foreach (Flug f in e.OldItems)
        { // Als zu löschen markieren (wirkt auf beide Listen)
          f.MarkAsDeleted();
          // Für direkten DB-Zugriff: Löschanweisung an Kontext senden
          if (this.C_Modus.Text == "DBDirekt") DB.DeleteObject(f);
        }
      }

      if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
      {
        foreach (Flug f in e.NewItems)
        { // Auch der Originalliste anfügen, damit diese alle Änderungen enthält
          fluegeKomplett.Add(f);
          // Für direkten DB-Zugriff: An Kontext anfügen
          if (this.C_Modus.Text == "DBDirekt") DB.Flug.AddObject(f);
        }
      }
    }

    /// <summary>
    /// Speichern der geänderten Flüge
    /// </summary>
    private void C_Speichern_Click(object sender, RoutedEventArgs e)
    {
      if (fluegeKomplett == null) return; // keine Daten geladen!

      // Änderungen anzeigen und nachfragen
      if (MessageBox.Show("Möchten Sie folgende Änderungen speichern?\n" + String.Format("Client: Geändert: {0} Neu: {1} Gelöscht: {2}", (from f in fluegeAktuelleAnsicht where f.ChangeTracker.State == ObjectState.Modified select f).Count(), (from f in fluegeKomplett where f.ChangeTracker.State == ObjectState.Added select f).Count(), (from f in fluegeKomplett where f.ChangeTracker.State == ObjectState.Deleted select f).Count()), "Nachfrage", MessageBoxButton.YesNo) == MessageBoxResult.No) return;


      string Ergebnis = "";
      // DB DIREKT
      if (this.C_Modus.Text == "DBDirekt")
      {
        //DB.DetectChanges(); nur, wenn nicht StartTracking()!
        Ergebnis = DB.SaveChanges().ToString();
      }
      // BL
      if (this.C_Modus.Text == "BL")
        Ergebnis = BL.SaveFluege(fluegeKomplett.ToList()).ToString();

      // APP SERVER
      if (this.C_Modus.Text == "Service")
      {
        // nur geänderte Objekte an den Server senden (vgl. GetChanges() im DataSet!)
        ObservableCollection<Flug> fluegeNurGeaenderte = new ObservableCollection<Flug>(
          (from f in fluegeKomplett where f.ChangeTracker.State != ObjectState.Unchanged 
           select f).ToList());
        // Webservice aufufen
        Ergebnis = Dienst.SaveFluege(fluegeNurGeaenderte);
      }

      // Acceptchanges muss man für jedes Objekt nun einzeln aufrufen !!!
      foreach (Flug f in fluegeAktuelleAnsicht)
      {
        f.AcceptChanges();
      }

      // Originalliste aktualisieren, damit die gelöschten dort verschwinden
      fluegeKomplett = new ObservableCollection<Flug>(fluegeAktuelleAnsicht);

      // Status anzeigen
      SetStatus("Gespeichert in der Datenbank: " + Ergebnis);
    }

    private void C_Test_Click(object sender, RoutedEventArgs e)
    {
      // APP SERVER
      MessageBox.Show(Dienst.Hello("Holger"), "Verbindungstest");
    }

    private void flugDataGrid_RowEditEnding(object sender, DataGridRowEditEndingEventArgs e)
    {


    }

    private void flugDataGrid_PreparingCellForEdit(object sender, DataGridPreparingCellForEditEventArgs e)
    {

    }

    private void C_Modus_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      //if (this.C_Test != null)
      //this.C_Test.IsEnabled = (e.AddedItems. ("Service"));
      if (this.flugDataGrid == null) return;
      this.flugDataGrid.ItemsSource = null;
      this.fluegeKomplett = null;

    }

    private void C_Test_Initialized(object sender, EventArgs e)
    {

      //this.C_Test.IsEnabled = (this.C_Modus.Text == "Service");
    }
  }
}
