﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Services.Client;
using de.WWWings.ConsoleClient;
using de.WWWings.ConsoleClient.DatenDienst;

namespace WWWings_35SP1_ConsoleClient
{
 partial class DataServiceClientDemo
 {
  public static void Run()
  {

   Console.WriteLine("START!");
   DSClient_Lesen_URI();
   DSClient_Lesen_LINQ();
   DSClient_Lesen_Aendern();
   DSClient_Lesen_Anfuegen();
   DSClient_Lesen_Loeschen();


   Console.WriteLine("ENDE!");

   Console.ReadLine();

  }


  /// <summary>
  /// Abfragen von Datensätzen über einen Data Service in URL-Syntax
  /// </summary>
  private static void DSClient_Lesen_URI()
  {
   // Kontext erstellen
   de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities DB = new de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/"));

   // Abfrage
   IEnumerable<Flug> Fluege = DB.Execute<Flug>(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/Flug?$filter=Abflugort%20eq%20'Rom'%20and%20Zielort%20eq%20'Berlin'"));

   // Ausgabe
   foreach (Flug f in Fluege)
   {
    Console.WriteLine(f.FlugNr + " fliegt von " + f.Abflugort + " nach " + f.Zielort + " und hat " + f.FreiePlaetze + "!");
   }
  }


  /// <summary>
  /// Abfragen von Datensätzen über einen Data Service in LINQ-Syntax
  /// </summary>
  private static void DSClient_Lesen_LINQ()
  {
   // Kontext erstellen
   de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities DB = new de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/"));

   // LINQ-Abfrage
   var Fluege = from f in DB.Flug where f.Abflugort == "Rom" select f;

   Console.WriteLine(Fluege.ToList().Count);
   // Ausgabe
   foreach (Flug f in Fluege)
   {
    Console.WriteLine(f.FlugNr + " fliegt von " + f.Abflugort + " nach " + f.Zielort + " und hat " + f.FreiePlaetze + "!");
   }
  }


  /// <summary>
  /// Ändern von Datensätzen über einen Data Service
  /// </summary>
  private static void DSClient_Lesen_Aendern()
  {
   de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities DB = new de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/"));

   var Fluege = (from f in DB.Flug where f.Abflugort == "Rom" && f.FreiePlaetze < 0 select f);

   foreach (Flug f in Fluege)
   {
    Console.WriteLine(f.FlugNr + " fliegt von " + f.Abflugort + " nach " + f.Zielort + " und hat " + f.FreiePlaetze + "!");

    // Objekt ändern
    f.FreiePlaetze = 0;

    // Objekt als verändert markieren
    DB.UpdateObject(f);
   }
   
   Console.WriteLine("Speichern aller Änderungen...");
   // Alle Änderungen speichern
   DB.SaveChanges();
  }


  /// <summary>
  /// Löschen eines Datensatzes über einen Data Service
  /// </summary>
  private static void DSClient_Lesen_Loeschen()
  {
   de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities DB = new de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/"));

   // Zugriff auf einen bestimmten Flug
   var flug = (from f in DB.Flug where f.FlugNr == NEUEFLUGNR select f).FirstOrDefault();

   // Flug löschen, wenn er gefunden wurde
   if (flug != null)
   {
    Console.WriteLine("Flug wird gelöscht!");
    DB.DeleteObject(flug);
   }
   else
   {
    Console.WriteLine("Flug nicht gefunden!");
   }

   // Alle Änderungen speichern
   DB.SaveChanges();
  }

  const int NEUEFLUGNR = 99996;
  /// <summary>
  /// Anfügen eines neuen Datensatzes über einen Data Service
  /// </summary>
  private static void DSClient_Lesen_Anfuegen()
  {
   de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities DB = new de.WWWings.ConsoleClient.DatenDienst.WWWings6Entities(new Uri("http://localhost:88/Webservice/WWWingsDataService.svc/"));

   // === Einen Datensatz ergänzen
   Flug n = new Flug();
   n.FlugNr = NEUEFLUGNR;  // bitte auf Einmaligkeit des Keys achten!
   n.Abflugort = "Peking";
   n.Zielort = "Sydney";
   n.Plaetze = 250;
   n.Datum = DateTime.Now.AddDays(100);
   n.FreiePlaetze = 250;
   DB.AddObject("Flug", n);

   Console.WriteLine("Flug wird ergänzt...");
   // Alle Änderungen speichern
   DB.SaveChanges();
  }

 //private static void Buchungen()
 // {
 //  int FNR = 301;

 //  WWWIngsEntities data = new WWWIngsEntities("http://localhost:88/Webservice/WWWingsDataService.svc/");

 //  data.MergeOption = MergeOption.OverwriteChanges;

 //  GF_GebuchteFluege[] count1 = (from b in data.GF_GebuchteFluege select b).ToArray();
 //  Console.WriteLine("Anzahl Buchungen vorher: " + count1.Length);

 //  // Zwei neue Buchungen für Flug 101
 //  GF_GebuchteFluege buchung1 = new GF_GebuchteFluege();
 //  buchung1.GF_FlugNr = FNR;
 //  buchung1.GF_PS_ID = 1;
 //  buchung1.GF_Preis = 0;
 //  buchung1.GF_Klasse = "X";
 //  buchung1.GF_Buchungscode = 12345;
 //  data.AddObject("GF_GebuchteFluege", buchung1);

 //  GF_GebuchteFluege buchung2 = new GF_GebuchteFluege();
 //  buchung2.GF_FlugNr = FNR;
 //  buchung2.GF_PS_ID = 2;
 //  buchung2.GF_Preis = 0;
 //  buchung2.GF_Klasse = "X";
 //  buchung2.GF_Buchungscode = 12346;
 //  data.AddObject("GF_GebuchteFluege", buchung2);

 //  // Anzahl freier Plätze reduzieren
 //  Fluege flug = (from f in data.Fluege where f.FlugNr == FNR select f).SingleOrDefault();

 //  Console.WriteLine("Freie Plätze vorher: " + flug.FreiePlaetze);
 //  flug.FreiePlaetze -= 2;
 //  Console.WriteLine("Freie Plätze nacher: " + flug.FreiePlaetze);
 //  data.UpdateObject(flug);
 //  data.SaveChanges();

 //  GF_GebuchteFluege[] count2 = (from b in data.GF_GebuchteFluege select b).ToArray();
 //  Console.WriteLine("Anzahl Buchungen vorher: " + count2.Length);

 //  WWWIngsEntities data2 = new WWWIngsEntities("http://localhost:88/Webservice/WWWingsDataService.svc/");
 //  Fluege flug2 = (from f in data2.Fluege where f.FlugNr == FNR select f).SingleOrDefault();
 //  Console.WriteLine("Freie Plätze Kontrolle: " + flug2.FreiePlaetze);
 // }
 
 }


}



//' === Client für ADO.NET Data Service
// Sub Beispiel_ADONETDataService()

//  ' Clientseitigen Kontext erzeugen, der über REST-basierten Webservice mit dem serverseitigen Datenkontext kommuniziert
//  Dim DB As New WWWingsModel.CS_EF_WWWings(New Uri("http://localhost:8562/EADS_Webservice/WWWingsDataService.svc"))

//  ' LINQ-Abfrage, die in eine URL umgesetzt wird
//  Dim Fluege = From f In DB.Fluege Where CBool(f.FreiePlaetze < 10) And f.Abflugort = "Berlin" Select f

//  ' Zum Testen: Ausgabe der URL
//  Console.WriteLine(Fluege.ToString())

//  ' Ausgabe und ändern der Ergebnis
//  For Each f As WorldWideWingsModel.Fluege In Fluege

//   Console.WriteLine(f.FlugNr & " fliegt von " & f.Abflugort & " nach " & f.Zielort & " und hat " & f.FreiePlaetze.ToString() & " freie Plätze!")

//   ' === Einen Datensatz löschen
//   If (f.FlugNr = 10004) Then
//    DB.DeleteObject(f)
//   Else
//    ' === Alle anderen Datensätze ändern
//    f.FreiePlaetze -= 1
//    DB.UpdateObject(f)

//   End If

//  Next

//  ' === Einen Datensatz ergänzen
//  'Dim n As New WorldWideWingsModel.Fluege
//  'n.FlugNr = 99999 ' bitte auf Einmaligkeit des Keys achten!
//  'n.Abflugort = "Peking"
//  'n.Zielort = "Sydney"
//  'n.Plaetze = 250
//  'n.FreiePlaetze = 250
//  'DB.AddObject("Fluege", n)

//  ' === Alle Änderungen speichern
//  DB.SaveChanges()

// End Sub
