﻿using System;
using System.Linq;
using de.WWWings.Test.Modelle.LTS6;
using de.WWWings.Test;

namespace de.WWWingsInit
{
 class DBInit_V6
 {

  /// <summary>
  /// Menge von Flügen erzeugen
  /// </summary>
  /// 
  static string[] Flughaefen = { "Berlin", "Frankfurt", "München", "Hamburg", "Köln/Bonn", "Rom", "London", "Paris", "Mailand", "Prag", "Moskau", "New York", "Seattle", "Essen/Mülheim", "Kapstadt", "Madrid", "Oslo", "Dallas" };
  const int ANZFluege = 500;
  const int ANZPass = 200;

  public static void Init_DB()
  {


   LTS.HeadLine("DB Version 6 Initialisieren");





   de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db = new de.WWWings.Test.Modelle.LTS6.WWWings6DataContext();

   
   Console.WriteLine("Flüge zu Beginn: " + db.Flug.Count());
   Console.WriteLine("Buchungen zu Beginn: " + db.Flug_Passagier.Count());
   Console.WriteLine("Passagiere zu Beginn: " + db.Passagier.Count());
   // Alle Buchungen löschen!
   db.ExecuteCommand("Delete from Flug_Passagier");
   db.ExecuteCommand("Delete from Flug");
   db.ExecuteCommand("Delete from pilot");
   db.ExecuteCommand("Delete from Mitarbeiter");
   db.ExecuteCommand("Delete from Passagier");
   db.ExecuteCommand("Delete from Person");

   Console.WriteLine("Flüge nach Löschen: " + db.Flug.Count());
   Console.WriteLine("Buchungen nach Löschen: " + db.Flug_Passagier.Count());
   Console.WriteLine("Passagiere nach Löschen: " + db.Passagier.Count());

   Init_Passagiere(db);



   Init_Fluege(db);

   Init_Buchungen(db);

   Init_Mitarbeiter(db);

   Init_Piloten(db);
   Init_PilotenZuFluegen(db);

  }

  private static System.Random Init_Fluege(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   Console.WriteLine("Erzeuge Flüge...");
   var Passagiere = from p in db.Passagier select p;
   System.Random rnd = new Random();
   var Fluege = from f in db.Flug select f;
   int Start = 100;

   for (int i = Start; i < (Start + ANZFluege); i++)
   {

    if (i % 100 == 0) Console.WriteLine(i);
    de.WWWings.Test.Modelle.LTS6.Flug FlugNeu = new de.WWWings.Test.Modelle.LTS6.Flug();
    FlugNeu.FlugNr = i;
    FlugNeu.Abflugort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)];
    FlugNeu.Zielort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)]; ;
    if (FlugNeu.Abflugort == FlugNeu.Zielort) { i--; continue; } // Keine Rundflüge!
    FlugNeu.FreiePlaetze = Convert.ToInt16(new System.Random(i).Next(250));
    FlugNeu.Plaetze = 250;
    FlugNeu.Datum = DateTime.Now.AddDays((double)FlugNeu.FreiePlaetze).AddMinutes((double)FlugNeu.FreiePlaetze * 7);
    FlugNeu.Ankunft = FlugNeu.Datum.AddHours(2);
    FlugNeu.NichtRaucherFlug = true;


    db.Flug.InsertOnSubmit(FlugNeu);
    try
    {
     db.SubmitChanges();
    }
    catch (Exception ex)
    {
     Console.WriteLine("   " + FlugNeu.FlugNr + ":" + FlugNeu.Abflugort + " -> " + FlugNeu.Zielort + ": " + ex.Message.ToString());

    }

   }
   Console.WriteLine("Flüge nach dem Einfügen: " + db.Flug.Count());
   return rnd;
  }

  private static void Init_Buchungen(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   int k = 0;
   System.Random rnd5 = new Random();
   Console.WriteLine("Erzeuge Buchungen...");
   var Fluege2 = from f in db.Flug select f;

   Passagier[] PassagierArray = db.Passagier.ToArray();
   foreach (Flug f in Fluege2)
   {
    k++;
    if (k % 100 == 0) Console.WriteLine(k);

    for (int j = 1; j < 10; j++)
    {
     Random rnd2 = new System.Random(System.DateTime.Now.Millisecond);
     // Wähle Zufällig einen Passagier
     Passagier p = PassagierArray[Convert.ToInt16(rnd5.Next(PassagierArray.Count()))];
     //Console.WriteLine("   " + p.ID);
     // Buchung anlegen
     Flug_Passagier b1 = new Flug_Passagier();
     b1.Flug = f;
     b1.Passagier = p;
     db.Flug_Passagier.InsertOnSubmit(b1);


     try // Weil doppelte möglich sind durch Zufallsauswahl!
     {
      db.SubmitChanges();
     }
     catch (Exception)
     {
      db.Flug_Passagier.DeleteOnSubmit(b1);
     }
    }

    

   }
   Console.WriteLine("Buchungen nach dem Einfügen: " + db.Flug_Passagier.Count());
  }

  private static void Init_PilotenZuFluegen(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   Pilot[] Piloten = db.Pilot.ToArray();
   System.Random rnd4 = new Random();
   Console.WriteLine("Piloten zu Flügen...");
   foreach (Flug f in db.Flug)
   {
    f.Pilot_PersonID = Piloten[rnd4.Next(db.Pilot.Count() - 1)].PersonID;
   }
   db.SubmitChanges();
   Console.WriteLine("Piloten zugeordnet für Flüge!");
  }

  private static void Init_Piloten(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   Console.WriteLine("Erzeuge Piloten...");
   System.Random rnd3 = new Random();
   // Alle Mitarbeiter > 250 werden zu Mitarbeitern
   foreach (Person p in db.Person.Where(pe => pe.PersonID >= (ANZPass *0.75)))
   {
    Pilot pi = new Pilot();
    pi.PersonID = p.PersonID;
    pi.FlugscheinSeit = DateTime.Now.AddDays(-rnd3.Next(2000));
    pi.Flugscheintyp = 'A';
    pi.Flugschule = "Essen";
    pi.Flugstunden = rnd3.Next(5000);
    db.Pilot.InsertOnSubmit(pi);
   }
   db.SubmitChanges();
   Console.WriteLine("Piloten nach dem Einfügen: " + db.Pilot.Count());
  }

  private static void Init_Mitarbeiter(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   Person ErstePerson = null;
   Console.WriteLine("Erzeuge Mitarbeiter...");
   System.Random rnd2 = new Random();
   // Alle Personen > 200 werden zu Mitarbeitern
   foreach (Person p in db.Person.Where(pe => pe.PersonID >= (ANZPass/2)))
   {
    if (ErstePerson == null) ErstePerson = p;
    Mitarbeiter mi = new Mitarbeiter();
    mi.PersonID = p.PersonID;
    mi.Einstellungsdatum = DateTime.Now.AddDays(-rnd2.Next(1000));
    mi.VorgesetzterPersonID = ErstePerson.PersonID;
    mi.MitarbeiterNr = p.PersonID;
    db.Mitarbeiter.InsertOnSubmit(mi);
    db.SubmitChanges();

   }
   Console.WriteLine("Mitarbeiter nach dem Einfügen: " + db.Mitarbeiter.Count());
  }

  private static void Init_Passagiere(de.WWWings.Test.Modelle.LTS6.WWWings6DataContext db)
  {
   // Passagiere Anlegen
   Console.WriteLine("Erzeuge Passagiere...");
   // Quelle: http://de.wikipedia.org/wiki/Liste_der_h%C3%A4ufigsten_Familiennamen_in_Deutschland

   string[] Nachnamen = { "Müller", "Schmidt", "Schneider", "Fischer", "Weber", "Meyer", "Wagner", "Becker", "Schulz", "Hoffmann", "Schäfer", "Koch", "Bauer", "Richter", "Klein", "Wolf", "Schröder", "Neumann", "Schwarz", "Zimmermann" };
   // Quelle: http://www.beliebte-vornamen.de/3467-alle-spitzenreiter.htm

   string[] Vornamen = { "Leon", "Hannah", "Lukas", "Anna", "Leonie", "Marie", "Niklas", "Sarah", "Jan", "Laura", "Julia", "Lisa", "Kevin" };


   // oder: Politiker-Namenskarusell :-)
   //string[] Nachnamen = { "Gysi", "Stoiber", "Koch", "Steinmeyer", "Schröder", "Merkel", "Westerwelle", "Beck", "Lafontaine", "Trittin", "Roth" };
   //string[] Vornamen = { "Edmund", "Olaf", "Roland", "Gerhard", "Angela", "Joschka", "Guido", "Gregor", "Kurt", "Frank-Walter", "Oskar", "Jürgen", "Claudia" };
   System.Random rnd2 = new Random();


   //DBUtil.ClearTable("GF_GebuchteFluege");
   //DBUtil.ClearTable("Passagier");


   string[] Staedte = { "Berlin", "Hamburg", "München", "Köln", "Frankfurt", "Stuttgart", "Dortmund", "Essen", "Düsseldorf", "Bremen", "Hannover", "Leipzig", "Dresden", "Nürnberg", "Duisburg", "Bochum", "Wuppertal", "Bielefeld", "Bonn", "Mannheim" };
   char[] KundenStatus = { 'A', 'B', 'C' };

   for (int PNummer = 1; PNummer <= ANZPass; PNummer++)
   {
    string Vorname = Vornamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    string Nachname = Nachnamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    Person pe = new Person();
    pe.Name = Nachname;
    pe.Vorname = Vorname;
    pe.Geburtstag = new DateTime(1940, 1, 1).AddDays(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(20000)));
    pe.PersonID = PNummer;
    pe.Land = "DE";
    pe.Stadt = Staedte[rnd2.Next(0, Staedte.Length - 1)];
    pe.EMail = Vorname.Substring(0, 1) + "." + Nachname + "@world-wide-wings.de";
    db.Person.InsertOnSubmit(pe);

    if (PNummer < (ANZPass / 2))
    {
     Passagier ps = new Passagier();
     ps.PersonID = PNummer;
     ps.KundeSeit = pe.Geburtstag.Value.AddHours(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(5000)));
     ps.KundenStatus = KundenStatus[rnd2.Next(2)];
     db.Passagier.InsertOnSubmit(ps);
    }

    db.SubmitChanges();

    System.Console.WriteLine(PNummer + ": " + Vorname + " " + Nachname);

   }
   Console.WriteLine("Passagiere nach dem Anfügen: " + db.Passagier.Count<Passagier>());
  }

 }
}
