﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data.Linq.Provider;
using System.Data.Linq;
using System.Diagnostics;

namespace de.WWWings.Test
{
 class LTO_Prozesse
 {
  public static void run()
  {

   Demo_LTO_Prozesse();
   //Demo_Prozesse_Filter_MethodSyntax();

   Console.ReadLine();
  }


  // --------------------------------------------


  private static void Demo_LTO_Prozesse()
  {
   Process[] Prozesse = Process.GetProcesses();

   Process p;
   long i;
   double d;

   string s = "26. Geben Sie alle Prozesse aus, die weniger als 3.000.000 Bytes Speicher verbrauchen.";
   var Ergebnis =
       from n in Prozesse
       where n.WorkingSet64 < 3000000
       select n;
   Print(Ergebnis, s);

   s = "27. Geben Sie alle Prozesse aus, die weniger als 3.000.000 Bytes Speicher verbrauchen. Sortieren Sie die Liste aufsteigend nach Speicherverbrauch.";
   Ergebnis =
   from n in Prozesse
   where n.WorkingSet64 < 3000000
   orderby n.WorkingSet64 // optional
   select n;
   Print(Ergebnis, s);

   s = "28. Geben Sie alle Prozesse aus, die weniger als 3.000.000 Bytes Speicher verbrauchen. Sortieren Sie die Liste absteigend nach Speicherverbrauch.";
   Ergebnis =
   from n in Prozesse
   where n.WorkingSet64 < 3000000
   orderby n.WorkingSet64 descending // optional
   select n;
   Print(Ergebnis, s);

   s = "29. Geben Sie die Prozesse aus. Eliminieren Sie alle Duplikate.";
   var Ergebnis6 =
        (from n in Prozesse
         select n.ProcessName).Distinct();
   Print(Ergebnis6, s);

   s = "30. Geben Sie den vierten bis achten Prozess aus in der nach Speicherverbrauch aufsteigend sortierten Liste aller Prozesse, die mehr als 1.000.000 Bytes verbrauchen.";
   Ergebnis =
       (from n in Prozesse
        where n.WorkingSet64 > 1000000
        orderby n.WorkingSet64
        select n).Skip(3).Take(4);
   Print(Ergebnis, s);

   s = "31. Geben Sie den ersten Prozess aus in der nach Speicherverbrauch aufsteigend sortierten Liste aller Prozesse, die mehr als 1.000.000 Bytes verbrauchen.";
   p =
      (from n in Prozesse
       where n.WorkingSet64 > 1000000
       orderby n.WorkingSet64
       select n).First();
   Print(p, s);

   s = "32. Geben Sie letzten Prozess aus in der nach Speicherverbrauch aufsteigend sortierten Liste aller Prozesse, die mehr als 1.000.000 Bytes verbrauchen.";
   p =
      (from n in Prozesse
       where n.WorkingSet64 > 1000000
       orderby n.WorkingSet64
       select n).Last();
   Print(p, s);

   s = "33. Geben Sie den 10. Prozess aus in der nach Speicherverbrauch aufsteigend sortierten Liste aller Prozesse, die mehr als 1.000.000 Bytes verbrauchen.";
   p =
      (from n in Prozesse
       where n.WorkingSet64 > 1000000
       orderby n.WorkingSet64
       select n).ElementAt(9);
   Print(p, s);

   s = "34. Geben Sie den 150. Prozess aus! (Fangen Sie den Fehler ab!)";
   p =
      (from n in Prozesse
       select n).ElementAtOrDefault(149);
   Print(p, s);

   s = "35. Geben Sie Anzahl der Prozesse aus (mit einem LINQ-Statement!)";
   i =
      (from n in Prozesse
       select n).Count();
   Print(i, s);


   s = "36. Geben Sie nur den niedrigsten Speicherverbrauch aus";
   i =
      (from n in Prozesse
       select n.WorkingSet64).Min(); // Lösung ohne Lambda
   Print(i, s);

   s = "37. Geben Sie nur den höchsten Speicherverbrauch aus";
   i =
      (from n in Prozesse
       select n).Max(n => n.WorkingSet64); // Lösung mit Lambda
   Print(i, s);

   s = "38. Geben Sie den durchschnittlichen Speicherverbrauch aus";
   d =
      (from n in Prozesse
       select n).Average(n => n.WorkingSet64);
   Print(i, s);

   s = "39. Geben Sie die Summe des Speicherverbrauchs aus";
   i =
      (from n in Prozesse
       select n).Sum(n => n.WorkingSet64);
   Print(i, s);

   s = "40. Gruppieren Sie die Prozesse nach Namen.";
   IEnumerable<IGrouping<string, Process>> GruppeErgebnis =
       (from n in Prozesse
        group n by n.ProcessName);
   Print(GruppeErgebnis, s);

   s = "41. Geben Sie die Häufigkeit eines jeden Processnamens aus!";
   IDictionary<string, int> GruppeHaeufigkeit =
         (from n in Prozesse
          group n by n.ProcessName into g
          select new { Name = g.Key, AnzProzess = g.Count() }
          ).ToDictionary(y => y.Name, y => y.AnzProzess);
   Print(GruppeHaeufigkeit, s);

   s = "42. Starten Sie einen neuen Process (Notepad) und ermitteln Sie, durch einen Vergleich der Prozessliste vorher und nachher, welche Prozesse neu hinzugekommen sind. (Geben Sie die Process-ID und den Processnamen aus!)";
   Process neupro = Process.Start(@"C:\Windows\notepad.exe");
   neupro.WaitForInputIdle();
   Process[] Prozesse2 = Process.GetProcesses();

   //Print((from p1 in Prozesse where p1.ProcessName=="notepad" select p1), "Test");
   //Print((from p2 in Prozesse2 where p2.ProcessName=="notepad" select p2), "Test");
   IEnumerable<int> ProzessListe = (from n2 in Prozesse2 select n2.Id).Except(from n in Prozesse select n.Id);
   Print(ProzessListe, s);

   //var ProzessListe2 = from p in System.Diagnostics.Process.GetProcesses() select p.ProcessName;

   s = "43. Listen Sie die Prozesse mit ihren Threads auf.";
   var ProzesseMitThreads =
      (from n in Prozesse
       select new { n, n.Threads }
       );
   HeadLine(s);
   foreach (var x in ProzesseMitThreads)
   {
    Console.WriteLine(x.n);
    try
    {
     foreach (ProcessThread y in x.Threads)
     {
      Console.WriteLine(y.StartTime);
     }
    }
    catch (Exception)
    {
    }
   }

   s = "44. Geben Sie zu jedem Prozess die Anzahl der Threads aus!";
   var ProzesseMitThreadCount =
    (from n in Prozesse
     where n.Id > 10
     select new { n, n.Threads.Count }
     );


   HeadLine(s);
   foreach (var m in ProzesseMitThreadCount)
   {
    Console.WriteLine(m.n + ":" + m.Count);
   }

   s = "45. Geben Sie die Prozesse aus, die mehr als 10 Threads haben!";
   Ergebnis =
      (from n in Prozesse
       where n.Threads.Count > 10
       select n);
   Print(Ergebnis, s);

   s = "46. Geben Sie den/die Prozess(e) aus, der/die meisten Threads hat!";
   Ergebnis = (from n in Prozesse where n.Threads.Count == Prozesse.Max(x => x.Threads.Count) select n);
   Print(Ergebnis, s);
  }

  // --------------------------------------------------------------------

  private static void Print(IDictionary<string, int> gruppe, string s)
  {
   HeadLine(s);
   foreach (var x in gruppe)
   {
    Console.WriteLine(x.Key + ": " + x.Value);
   }
  }


  private static void Print(IEnumerable<Process> lowNums, string s)
  {
   HeadLine(s);
   foreach (Process x in lowNums)
   {
    PrintProcess(x);
   }
  }

  private static void Print(IEnumerable<string> lowNums, string s)
  {
   HeadLine(s);
   foreach (string x in lowNums)
   {
    Console.WriteLine(x);
   }
  }


  private static void Print(IEnumerable<int> lowNums, string s)
  {
   HeadLine(s);
   foreach (int x in lowNums)
   {
    Console.WriteLine(x);
   }
  }


  private static void Print(IEnumerable<IGrouping<string, Process>> Gruppen, string s)
  {
   HeadLine(s);
   foreach (IGrouping<string, Process> x in Gruppen)
   {
    Console.WriteLine("---- " + x.Key);
    foreach (Process y in x)
    {
     PrintProcess(y);
    }
   }
  }

  private static void Print(Process x, string s)
  {
   HeadLine(s);
   if (x == null) { Console.WriteLine("---"); return; }
   PrintProcess(x);

  }

  private static void PrintProcess(Process x)
  {
   Console.WriteLine(x.ProcessName + ": " + x.WorkingSet64 + " Threads: " + x.Threads.Count);
  }


  private static void Print(long x, string s)
  {
   HeadLine(s);

   Console.WriteLine(x);

  }
  private static void HeadLine(string s)
  {
   Console.ForegroundColor = ConsoleColor.Yellow;
   Console.WriteLine(s);
   Console.ForegroundColor = ConsoleColor.Gray;

   System.IO.StreamWriter sw = new System.IO.StreamWriter("Fragen.txt", true);
   sw.WriteLine(s);
   sw.Close();

  }

  //private static void Print(IEnumerable<int> lowNums, string s)
  //{
  // Console.ForegroundColor = ConsoleColor.Yellow;
  // Console.WriteLine(s);
  // Console.ForegroundColor = ConsoleColor.Gray;
  // foreach (var x in lowNums)
  // {
  //  Console.WriteLine(x);
  // }
  //}




 }
}
