﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Timers;
using System.Diagnostics;

namespace NET4_CS
{

 /// <summary>
 /// Einstaz von PLINQ
 /// </summary>
 class Demo_PLINQ
 {

  /// <summary>
  /// Vergleich einer einfachen Frage mit und ohne PLINQ
  /// </summary>
  public static void LTO_ohne_PLINQ()
  {
   int[] Zahlen1 = { 15, 4, 11, 3, 19, 8, 16, 7, 12, 5, 9, 20, 1, 4, 8, 13, 14, 4, 1 };
   string s = "Geben Sie die Zahlen aus, die kleiner als 10 sind.";

   Stopwatch t = new Stopwatch();
   t.Start();
   var Ergebnis =
       from n in Zahlen1
       where n < 10
       orderby n
       select n;

   var Liste = Ergebnis.ToList();

   t.Stop();
   Console.WriteLine("Anzahl = " + Liste.Count());
   Console.WriteLine("Abfrage ohne PLINQ  = " + t.ElapsedMilliseconds);
  }


  /// <summary>
  /// Vergleich einer einfachen Frage mit und ohne PLINQ
  /// </summary>
  public static void LTO_mit_PLINQ()
  {
   int[] Zahlen = { 15, 4, 11, 3, 19, 8, 16, 7, 12, 5, 9, 20, 1, 4, 8, 13, 14, 4, 1 };
   string s = "Geben Sie die Zahlen aus, die kleiner als 10 sind.";

   Stopwatch t = new Stopwatch();
   t.Start();
   var Ergebnis =
       from n in Zahlen.AsParallel()
       where n < 10
       orderby n
       select n;

   var Liste = Ergebnis.ToList();

   t.Stop();
   Console.WriteLine("Anzahl = " + Liste.Count());
   Console.WriteLine("Abfrage mit PLINQ  = " + t.ElapsedMilliseconds);
  }




  /// <summary>
  /// Massendaten filtern und sortieren mit PLINQ
  /// </summary>
  public static void LTOMassendaten_ohne_PLINQ()
  {
   
   long AnzZahlen = 100000;
   System.Random rnd = new Random(DateTime.Now.Year);
   List<long> Zahlen = new List<long>();
   for (int i = 1; i <= AnzZahlen; i++) Zahlen.Add(rnd.Next(100));


   long Summe = 0;
   Stopwatch t = new Stopwatch();
   t.Start();
   for (int w = 1; w <= 20; w++)
   {

    var q = (from x in Zahlen orderby x where x < 50 select x).ToList();
    Summe += q.Count();
   }
   t.Stop();
   Console.WriteLine("Summe: " + Summe);
   Console.WriteLine("Ohne PLINQ  = " + t.ElapsedMilliseconds);
  }

  /// <summary>
  /// Massendaten filtern und sortieren ohne PLINQ
  /// </summary>
  public static void LTOMassendaten_mit_PLINQ()
  {

   long AnzZahlen = 100000;
   System.Random rnd = new Random(DateTime.Now.Year);
   List<long> Zahlen = new List<long>();
   for (int i = 1; i <= AnzZahlen; i++) Zahlen.Add(rnd.Next(100));

   long Summe = 0;
   Stopwatch t = new Stopwatch();
   t.Start();
   for (int w = 1; w <= 20; w++)
   {
    var q = (from x in Zahlen.AsParallel() where x < 50  orderby x select x).ToList();
    Summe += q.Count();
   }
   t.Stop();
   Console.WriteLine("Summe: " + Summe);
   Console.WriteLine("Mit PLINQ  = " + t.ElapsedMilliseconds);

  }


 }
}
