﻿using System;
using System.Linq;
using de.WWWings.Test.Modelle.LTS4;
using System.Data.Linq;

namespace de.WWWings.Test.LINQ
{

 public class Buchungsbeispiel_DB4_LTS
 {
  DatenbankDataContext db = new DatenbankDataContext();

  public int NewBuchung()
  {
   // Eager Loading aktivieren
   DataLoadOptions dlo = new DataLoadOptions();
   dlo.LoadWith<FL_Fluege>(f => f.GF_GebuchteFlueges);
   dlo.LoadWith<GF_GebuchteFluege>(b => b.PS_Passagier);
   dlo.LoadWith<PS_Passagier>(p => p.PE_Person);
   db.LoadOptions = dlo;

   int FNummer = 111;
   int PNummer = 2746;
   int Platzanzahl = 2;
   int Buchungscode = Convert.ToInt32(FNummer.ToString() + PNummer.ToString());

   // Transaktion, nur erfolgreich wenn Platzanzahl reduziert und Buchung erstellt!
   using (System.Transactions.TransactionScope t = new System.Transactions.TransactionScope())
   {
    try
    {
     // -------------------------------------------------
     Program.Headline("--- Eingabedaten:");
     //Passagier
     PS_Passagier EinPassagier = (from p in db.PS_Passagiers where p.PS_ID == PNummer select p).SingleOrDefault();
     Console.WriteLine("Passagier: " + EinPassagier.PS_ID + " " + EinPassagier.PE_Person.PE_Vorname + " " + EinPassagier.PE_Person.PE_Name);
     // Flug
     FL_Fluege EinFlug = (from f in db.FL_Flueges where f.FL_FlugNr == FNummer select f).SingleOrDefault();
     Console.WriteLine("Flug: " + EinFlug.FL_FlugNr + " von " + EinFlug.FL_Abflugort + " nach " + EinFlug.FL_Zielort);

     // -------------------------------------------------
     Program.Headline("--- Bestehende Buchungen:");
     foreach (GF_GebuchteFluege b in EinFlug.GF_GebuchteFlueges)
     {
      Console.WriteLine(b.PS_Passagier.PS_ID + ": " + b.PS_Passagier.PE_Person.PE_Vorname + " " + b.PS_Passagier.PE_Person.PE_Name);
     }

     // -------------------------------------------------
     Program.Headline("--- Neue Buchung:");

     // Platzanzahl reduzieren
     Console.WriteLine("Platzanzahl reduzieren...");
     if (EinFlug != null)
     {
      if (EinFlug.FL_FreiePlaetze >= Platzanzahl)
      {
       Console.WriteLine("Platzanzahl vorher: " + EinFlug.FL_FreiePlaetze.ToString());
       EinFlug.FL_FreiePlaetze -= Platzanzahl;
       db.SubmitChanges(); // Speichern!!!
       Console.WriteLine("Platzanzahl nun: " + EinFlug.FL_FreiePlaetze.ToString());
      }
      else
      {
       return 0; // Kein Platz vorhanden!
      }
      // Buchung erstellen
      // mit LINQ-to-SQL aber über SQL:
      //db.ExecuteCommand("Insert into GF_GebuchteFluege (GF_FL_FlugNr,GF_PS_ID, GF_Buchungscode) values ( " +  FNummer + "," + PNummer + "," + Buchungscode + ")");
      // mit LINQ-to-SQL;
      Console.WriteLine("Buchung erstellen...");
      GF_GebuchteFluege b = new GF_GebuchteFluege();
      //b.GF_FL_FlugNr = FNummer;
      b.FL_Fluege = EinFlug;
      //b.GF_PS_ID = PNummer;
      b.PS_Passagier = EinPassagier;
      b.GF_Buchungscode = Buchungscode;
      db.GF_GebuchteFlueges.InsertOnSubmit(b); // Einfügen
      db.SubmitChanges(); // Speichern
     }
    } // End Try
    catch (Exception ex)
    {
     Console.WriteLine("Fehler: " + ex.Message);
     return 0;
    }
    // Transaktion erfolgreich
    t.Complete();
   } // End Transaktion
   Console.WriteLine("Abgeschlossen. Buchungscode: " + Buchungscode.ToString());
   return Buchungscode;
  }
 }

}

