﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

namespace de.WWWings.Test.LINQExtensions
{

 internal static class WWWingsCollectionExtensions
 {
  // --- Erweiterungsmethode für IEnumerable
  public static void Print(this IEnumerable Menge, ConsoleColor Farbe)
  {
   ConsoleColor VorherigeFarbe = Console.ForegroundColor;
   Console.ForegroundColor = Farbe;
   foreach (object o in Menge)
    Console.WriteLine(o.ToString());
   Console.ForegroundColor = VorherigeFarbe;
  }
 }


 /// <summary>
 /// Erweiterung für LINQ-Projektionen 
 /// (C) www.IT-Visions.de Holger Schwichtenberg 2008 
 /// </summary>
 internal static class WWWingsProjectionExtensions
 {

  /// <summary>
  /// Konvertierung einer Menge eines (anonymen) Typs in einen anderen Typ 
  /// durch Abbildung gleichnamiger Attribute
  /// </summary>
  public static IEnumerable<T> Convert<T>(this IEnumerable Menge)
   where T : new()
  {
   // Neue Menge
   List<T> Result = new List<T>();
   // Duplizieren der enthaltenen Objekte
   foreach (object o in Menge)
   {
    T e = CopyTo<T>(o);
    Result.Add(e);
   }
   return Result;
  }

  /// <summary>
  /// Konvertierung einer Menge eines (anonymen) Typs in einen anderen Typ 
  /// durch Abbildung gleichnamiger Attribute und Anhängen an LTS
  /// </summary>
  public static IEnumerable<T> Convert<T>(this IEnumerable Menge, System.Data.Linq.ITable Table)
   where T : new()
  {
   // Neue Menge
   List<T> Result = new List<T>();
   // Duplizieren der enthaltenen Objekte
   foreach (object o in Menge)
  { 
    T e = CopyTo<T>(o);
    Result.Add(e);
   }
   // Anfügen der neuen Menge an den Datenkontext
   Table.AttachAll(Result, false);
   return Result;
  }

  /// <summary>
  /// Kopieren gleichnamiger Attribut auf ein anderes Objekt
  /// </summary>
  public static T CopyTo<T>(object from)
    where T : new()
  {
   T to = new T();
   return CopyTo<T>(from, to);
  }

  /// <summary>
  /// Kopieren gleichnamiger Attribut auf ein anderes Objekt
  /// </summary>
  public static T CopyTo<T>(object from, T to)
     where T : new()
  {
   Type FromType = from.GetType();
   Type ToType = to.GetType();
   
   // Kopieren der Fields
   foreach (FieldInfo f in FromType.GetFields())
   {
    FieldInfo t = ToType.GetField(f.Name);
    if (t != null)
    {
     t.SetValue(to, f.GetValue(from));
    }
   }

   // Kopieren der Properties
   foreach (PropertyInfo f in FromType.GetProperties())
   {
    object[] Empty = new object[0];
    PropertyInfo t = ToType.GetProperty(f.Name);
    if (t != null)
    {
     //Console.WriteLine(f.GetValue(from, Empty));
     t.SetValue(to, f.GetValue(from, Empty), Empty);
    }
   }

   return to;
  }
 }
}

