﻿using System;
using System.Collections.Generic;
using System.Text;
using de.WWWings.Library;
using System.Collections;

namespace NET3.SpracheCSharp.Demo_Sprachfeatures
{
 public partial class Vorstandsmitglied
 {




  // Automatic Properties
  public string Name { get; set; }
  public string Aufgabengebiet { get; set; }
  public int Alter { get; set; }
  public string Ort;


  public override string ToString()
  {
   OnToString();
   return Name;
  }

  // Partielle Methode - Deklaration
  partial void OnToString();
 }

 public partial class Vorstandsmitglied
 {
  // Partielle Methode - Implementierung
  partial void OnToString()
  {
   Console.WriteLine("ToString aufgerufen!");
  }
 }

 public class Fluggesellschaft_Client
 {

  public static void RunLambda()
  {

   // Lambda-Ausdrücke deklarieren
   Func<int, int> f1 = x => x + 1;
   Func<string, string> f2 = s => s.ToUpper();
   Func<string, int> f3 = s => s.Length;
   Func<string, int, string> f4 = (s, i) => s.Substring(0, i);

   // Lambda-Ausdrücke verwenden
   Console.WriteLine(f1(10)); // ergibt 11
   Console.WriteLine(f2("World Wide Wings")); // ergibt WORLD WIDE WINGS
   Console.WriteLine(f3("World Wide Wings")); // ergibt 16
   Console.WriteLine(f4("World Wide Wings", 10)); // Ergibt "World Wide"

  }


  public static void run()
  {
   // Object Initializer
   Vorstandsmitglied MM = new Vorstandsmitglied() { Name = "Max Müller", Aufgabengebiet = "Flugbetrieb", Alter = 33 };
   Vorstandsmitglied HM = new Vorstandsmitglied() { Name = "Hans Meier", Aufgabengebiet = "Personal", Alter = 42 };
   Vorstandsmitglied HS = new Vorstandsmitglied() { Name = "Hubert Schmidt", Aufgabengebiet = "Marketing", Alter = 35, Ort = "Essen" };

   Vorstandsmitglied v1 = new Vorstandsmitglied();
   var v2 = new Vorstandsmitglied();

   // Collection Initializer
   List<Vorstandsmitglied> Vorstandsmitglieder = new List<Vorstandsmitglied> { HS, HM, MM };
  

   // Verwendung einer Erweiterungsmethode
   Vorstandsmitglieder.Print(ConsoleColor.DarkYellow);

   // Local Variable Type Inference für String
   var Heimatflughafen = "Essen/Mülheim";
   Console.WriteLine(Heimatflughafen.GetType().FullName);

   // Local Variable Type Inference für Int32
   var Anzahl = Vorstandsmitglieder.Count;
   Console.WriteLine(Anzahl.GetType().FullName);

   // Local Variable Type Inference für die Klasse Vorstandsmitglied
   var Vorstandschef = Vorstandsmitglieder[0];
   Console.WriteLine(Vorstandschef.GetType().FullName);

   // Anonyme Typen
   var Fluggesellschaft = new { Name = "World Wide Wings", Gruendungsdatum = new DateTime(2005, 01, 01), Vorstand = Vorstandsmitglieder };
   Console.WriteLine(Fluggesellschaft.GetType().FullName);

   // 2., gleich aufgebauter anonymer Typ
   var Flugzeugbauer = new { Name = "Strong Winds Corp.", Gruendungsdatum = new DateTime(1972, 08, 01), Vorstand = new List<Vorstandsmitglied>() };
   Console.WriteLine(Flugzeugbauer.GetType().FullName);



   // Prädikate klassische Schreibweise
   List<Vorstandsmitglied> JungeVorstandsmitglieder1 = Vorstandsmitglieder.FindAll(AuswahlJunge);
   Console.WriteLine("JungeVorstandsmitglieder: " + JungeVorstandsmitglieder1.Count);

   // Prädikate mit anonymen Methoden
   List<Vorstandsmitglied> JungeVorstandsmitglieder2 = Vorstandsmitglieder.FindAll(delegate(Vorstandsmitglied v) { return v.Alter < 40;  });
   Console.WriteLine("JungeVorstandsmitglieder: " + JungeVorstandsmitglieder2.Count);

   // Prädikate mit Lambda-Expression
   List<Vorstandsmitglied> JungeVorstandsmitglieder3 = Vorstandsmitglieder.FindAll(v => v.Alter < 40);
   Console.WriteLine("JungeVorstandsmitglieder: " + JungeVorstandsmitglieder3.Count);
  }

  // gehört zu Prädikat klassische Schreibweise!
  static public bool AuswahlJunge(Vorstandsmitglied v)
  {
   return (v.Alter < 40);
  }
 }


}
